@@ -44,6 +44,7 @@
 #include <string.h>
 #include <assert.h>
 #include <math.h>
+#include <time.h>
 
 #include "lib_linalg.h"
 #include "lib_optimization.h"
@@ -459,7 +460,6 @@ c3_regression_run_aio(struct FTparam * ftp, struct RegressOpts * ropts,
                       size_t N, const double * x, const double * y)
 {
 
-
     enum FTPARAM_ST structure = ft_param_extract_structure(ftp);
     struct SLMemManager * mem = sl_mem_manager_alloc(ftp->dim,N,ftp->nparams,structure);
     sl_mem_manager_check_structure(mem,ftp,x);
@@ -1238,15 +1238,17 @@ ft_regress_run(struct FTRegress * ftr, struct c3Opt * optimizer,
     assert (ftr->ftp != NULL);
     assert (ftr->regopts != NULL);
 
-    srand(ftr->seed);
+    /* printf("seed = %d\n", ftr->seed); */
+    /* srand(ftr->seed); */
     double param_norm = cblas_ddot(ftr->ftp->nparams,ftr->ftp->params,1,ftr->ftp->params,1);
+    /* printf("param_norm = %3.15E\n", param_norm); */
     if (fabs(param_norm) <= 1e-15){
         /* double yavg = 0.0; for (size_t ii = 0; ii < N; ii++){ yavg += y[ii];} */
         /* yavg /= (double) N; */
 
         /* size_t dim = ftr->dim; */
         /* ft_param_create_from_lin_ls(ftr->ftp,N,x,y,pow(1e-3, 1.0/dim)); */
-        ft_param_create_from_lin_ls(ftr->ftp,N,x,y,1e-2);
+        ft_param_create_from_lin_ls(ftr->ftp,N,x,y,1e-2, &(ftr->seed));
 
         
         /* ft_param_create_from_constant */
@@ -1343,6 +1345,14 @@ ft_regress_run_rankadapt(struct FTRegress * ftr,
         if (ftr->regopts->verbose > 0){
             printf("Rounded ranks: "); iprint_sz(ftr->dim+1,ftr_ranks);
         }
+
+        int allmax = 1;
+        for (size_t ii = 1; ii < ft->dim;ii++){
+            if (ranks[ii] < maxrank){
+                allmax = 0;                
+                break;
+            }
+        }
         
         kicked = 0;
 
@@ -1357,13 +1367,7 @@ ft_regress_run_rankadapt(struct FTRegress * ftr,
             }
         }
 
-        int allmax = 1;
-        for (size_t ii = 1; ii < ft->dim;ii++){
-            if (ranks[ii] < maxrank){
-                allmax = 0;                
-                break;
-            }
-        }
+
 
         /* printf("kicked == %zu\n",kicked); */
         if ((kicked == 0) || (allmax == 1)){
@@ -1612,9 +1616,10 @@ double cross_validate_run(struct CrossValidate * cv,
     double norm = 0.0;
 
     // keep the same starting parameters for each cv;
-    double * params = calloc_double(reg->ftp->nparams);
-    memmove(params,reg->ftp->params, reg->ftp->nparams * sizeof(double));
+    struct FTparam * ftp_original = ft_param_copy(reg->ftp);
+    ft_param_free(reg->ftp); reg->ftp = NULL;
     for (size_t ii = 0; ii < cv->kfold; ii++){
+        reg->ftp = ft_param_copy(ftp_original);
 
         if (ii == cv->kfold-1){
             batch = cv->N - start_num;
@@ -1663,8 +1668,10 @@ double cross_validate_run(struct CrossValidate * cv,
         start_num += batch;
 
         // reset parameters
-        ft_regress_update_params(reg,params);
+        ft_param_free(reg->ftp); reg->ftp = NULL;
+        reg->ftp = ftp_original;
     }
+
     if (cv->verbose > 1){
         printf("\t CV Err = %G, norm  = %G, relative_err = %G\n",erri/cv->N,norm/cv->N,erri/norm);
     }
@@ -1675,7 +1682,7 @@ double cross_validate_run(struct CrossValidate * cv,
     free(ytrain); ytrain = NULL;
     free(xtest); xtest = NULL;
     free(ytest); ytest = NULL;
-    free(params); params = NULL;
+    
 
     return err;
 }
