@@ -263,19 +263,30 @@ class FunctionTrain(object):
                              round_tol=1e-8, kickrank=5, maxiter=5, fiber_bounds=None):
 
 
+        if isinstance(init_rank, int):
+            use_rank = np.ones((self.dim+1),dtype=np.uint)
+            use_rank[1:self.dim] = init_rank
+        else:
+            assert(len(init_rank) == self.dim+1)
+            use_rank = init_rank.astype(dtype=np.uint)
+            
         start_fibers = c3.malloc_dd(self.dim)
 
         c3a, onedopts, low_opts, optnodes = self._build_approx_params(c3.CROSS)
 
 
         for ii in range(self.dim):
+            if ii == 0:
+                rr = int(use_rank[1])
+            else:
+                rr = int(use_rank[ii])
             if fiber_bounds is None:
                 c3.dd_row_linspace(start_fibers, ii, self.opts[ii]['lb'],
-                                   self.opts[ii]['ub'], init_rank)
+                                   self.opts[ii]['ub'], rr)
             else:
                 # print("should be here! ", fiber_bounds)
                 c3.dd_row_linspace(start_fibers, ii, fiber_bounds[ii][0],
-                                   fiber_bounds[ii][1], init_rank)
+                                   fiber_bounds[ii][1], rr)
 
         
             # c3.dd_row_linspace(start_fibers, ii, 0.8,
@@ -283,7 +294,8 @@ class FunctionTrain(object):
             
         # NEED TO FREE OPTNODES
         
-        c3.c3approx_init_cross(c3a, init_rank, verbose, start_fibers)
+        # c3.c3approx_init_cross(c3a, init_rank, verbose, start_fibers)
+        c3.c3approx_init_cross_het(c3a, use_rank, verbose, start_fibers)
         c3.c3approx_set_verbose(c3a, verbose)
         c3.c3approx_set_cross_tol(c3a, cross_tol)
         c3.c3approx_set_cross_maxiter(c3a, maxiter)
@@ -410,7 +422,7 @@ class FunctionTrain(object):
         elif (cvnparam is not None) and (cvregweight is not None) and (cvrank is None):
             cvgrid = c3.cv_opt_grid_init(2)
             c3.cv_opt_grid_add_param(cvgrid, "num_param", len(cvnparam), list(cvnparam))
-            c3.cv_opt_grid_add_param(cvgrid, "reg_weight", len(cvregeight), list(cvnparam))
+            c3.cv_opt_grid_add_param(cvgrid, "reg_weight", len(cvregweight), list(cvnparam))
         elif (cvnparam is not None) and (cvrank is not None):
             cvgrid = c3.cv_opt_grid_init(2)
             c3.cv_opt_grid_add_param(cvgrid, "rank", len(cvrank), list(cvrank))
@@ -438,9 +450,9 @@ class FunctionTrain(object):
         # print("Run regression")
         if seed is not None:
             c3.ft_regress_set_seed(reg, seed)
-            
+
         self.ft = c3.ft_regress_run(reg, optimizer, xdata.flatten(order='C'), yuse)
-        # print("Done!")
+
 
 
         if norm_ydata is True: # need to unnormalize
@@ -512,12 +524,27 @@ class FunctionTrain(object):
         c3.function_train_gradient_eval(self.ft, pt, grad_out)
         return grad_out
 
-    def round(self, eps=1e-14):
-        """ Round a FunctionTrain """
+    def hess_eval(self, pt):
+        """ Evaluate the hessian of a FunctionTrain """
 
+        hess_out = np.zeros((self.dim*self.dim))
+        hess = c3.function_train_hessian(self.ft)
+        c3.ft1d_array_eval2(hess, pt, hess_out)
+        c3.ft1d_array_free(hess)
+        return hess_out.reshape((self.dim, self.dim), order='F')
+
+    def round(self, eps=1e-14, maxrank_all=None):
+        """Round a FunctionTrain. """
+        
         c3a, onedopts, low_opts, optopts = self._build_approx_params()
         multiopts = c3.c3approx_get_approx_args(c3a)
-        ftc = c3.function_train_round(self.ft, eps, multiopts)
+
+        if maxrank_all is not None:
+            assert isinstance(maxrank_all, int)
+            ftc = c3.function_train_round_maxrank_all(self.ft, eps, multiopts, maxrank_all)
+        else:
+            ftc = c3.function_train_round(self.ft, eps, multiopts)
+            
         c3.function_train_free(self.ft)
         # c3.function_train_free(self.ft)
         self.ft = ftc
@@ -567,7 +594,7 @@ class FunctionTrain(object):
 
     def scale_and_shift(self, scale, shift, eps=0, c3_pointer=False):
 
-        c3a, onedopts, low_opts = self._build_approx_params()
+        c3a, onedopts, low_opts, optnodes = self._build_approx_params()
         multiopts = c3.c3approx_get_approx_args(c3a)
 
         ft1 = c3.function_train_copy(self.ft)
@@ -583,7 +610,7 @@ class FunctionTrain(object):
         c3.function_train_free(ft2)
 
         c3.function_train_round(ft_out.ft, eps, multiopts)
-        self._free_approx_params(c3a, onedopts, low_opts)
+        self._free_approx_params(c3a, onedopts, low_opts, optnodes)
 
         if c3_pointer is True:
             ft_ret = c3.function_train_copy(ft_out.ft)
@@ -628,7 +655,24 @@ class FunctionTrain(object):
             ft_out.round(eps=eps)
 
         return ft_out
-        
+
+    def laplace_op(self, op, eps=0.0):
+        # op should be computed using e.g., build_lp_operator
+        ft_out = FunctionTrain(self.dim)
+
+        c3a, onedopts, low_opts, opt_opts = self._build_approx_params(c3.REGRESS)
+        multiopts = c3.c3approx_get_approx_args(c3a)
+
+        opp = c3.operator_for_laplace_get_op(op)
+        ft_out.ft = c3.exact_laplace_op(self.ft, opp, multiopts)
+        ft_out.opts = copy.deepcopy(self.opts)
+
+        self._free_approx_params(c3a, onedopts, low_opts, opt_opts)
+
+        if eps > 0.0:
+            ft_out.round(eps=eps)
+
+        return ft_out
         
     def __del__(self):
         self.close()
@@ -640,8 +684,107 @@ class FunctionTrain(object):
             self.ft = None
 
 
+class TensorTrain(FunctionTrain):
+    """A tensor-train object based on the FunctionTrain
+    To achieve this effect, I use a Function train
+    with piecewise-linear approximation
 
+    Parameters
+    ----------
+    cores - a list of cores
+    """
+
+    # def __init__(self, cores):
+    #     self.cores = copy.deepcopy(cores)
+        
+    def __getitem__(self, *index):
+        x = np.array(*index)
+        return self.eval(x)
+    
+    def eval(self, xin):
+        assert isinstance(xin, np.ndarray)
+
+        if xin.ndim == 1:
+            left = self.cores[0][0, xin[0], :]
+            for ii in range(1, self.dim):
+                current = self.cores[ii][:, xin[ii], :]
+                # print("left.shape = ", left.shape)
+                # print("current.shape = ", current.shape)
+                left = np.einsum("k,kl->l", left, current)
+
+            left = np.squeeze(left, axis=-1)
+            return left
+        else:
+            left = self.cores[0][0, xin[:, 0], :]
+            for ii in range(1, self.dim):
+                current = self.cores[ii][:, xin[:, ii], :]
+                # print("left.shape = ", left.shape)
+                # print("current.shape = ", current.shape)
+                left = np.einsum("mk,kml->ml", left, current)
+
+            left = np.squeeze(left, axis=-1)
+            return left
+    
+    @classmethod
+    def cross_from_numpy_tensor(cls, tens, init_rank, verbose, adapt,
+                                maxrank=10, cross_tol=1e-8,
+                                round_tol=1e-8, kickrank=5, maxiter=5):
+        """Use cross approximation to build approximation of a tensor 
+        the keyword arguments are the same as those of cross approximation
+        """
+        
+        mode_sizes = tens.shape
+        dim = len(mode_sizes)
+        tt = cls(dim)
+        nodes = []
+        for ii in range(dim):
+            lb = float(0)
+            ub = float(mode_sizes[ii])
+            lin_elem_nodes = np.arange(lb, ub, 1.0)
+            nodes.append(lin_elem_nodes)
+            # print(lin_elem_nodes)
+            tt.set_dim_opts(ii, "linelm", lb, ub, lin_elem_nodes=lin_elem_nodes)
+
+        def func(x, param=None):
             
+            out = np.zeros((x.shape[0]))
+            xuse = np.copy(x).astype('int')
+            # print(xuse)
+            use_index = 0
+            for ii in range(xuse.shape[1]):
+                if xuse[1, ii] != xuse[0, ii]:
+                    use_index = ii
+                    break
+            
+            index = xuse[0, :].tolist()
+            index[use_index] = slice(None)
+            index = tuple(index)
+            return param[index]
+
+        
+        print("build!")
+        tt.build_approximation(func, tens, init_rank, verbose, adapt,
+                               maxrank=maxrank, cross_tol=cross_tol,
+                               round_tol=round_tol, kickrank=kickrank, maxiter=maxiter)
+
+
+        tt.cores = [None] * dim
+        ranks = tt.get_ranks()
+        for ii in range(dim):
+
+            core = np.zeros((ranks[ii], tens.shape[ii], ranks[ii+1]))
+            qmarray = c3.function_train_get_core(tt.ft, ii)
+            for jj in range(ranks[ii]):
+                for kk in range(ranks[ii+1]):
+                    func = GenericFunction(c3.qmarray_get_func(qmarray, jj, kk))
+                    core[jj, :, kk] = func.eval(nodes[ii])
+            tt.cores[ii] = core
+
+        return tt
+        
+    
+
+    
 class GenericFunction(object):
     """ Univariate Functions """
 
@@ -695,7 +838,7 @@ class SobolIndices(object):
         
     def __del__(self):
         self.close()
-        
+
     def close(self):
         if self.si is not None:
             c3.c3_sobol_sensitivity_free(self.si)
